#include <PID_v1.h>    // Controller LIB
#include <Encoder.h>   // Winkelencoder LIB

//Mechanical Sytem Properties
  //full pendulum rotation =8000 Encoder counts
#define rot_Encoder_A 20    // rotary Encoderpin A an Pin 20
#define rot_Encoder_B 21    // rotary Encoderpin B an Pin 21
Encoder rotEnc(rot_Encoder_A, rot_Encoder_B);   

//Variable Definitions

  //Pin Definitions
   byte pin_photoint=13;
   int RPWM_Output = 6; // Arduino PWM output pin 6; connect to IBT-2 pin 1 (RPWM)
   int LPWM_Output = 7; // Arduino PWM output pin 7; connect to IBT-2 pin 2 (LPWM)
 
   double limit_angle= 300; //limit at which it becomes impossible to catch the pendulum again if exeeded, active control of pendulum is stopped and cart is stopped if exeeded. Limit works in both direction (+ and -) at the equilibrium point
   
 //Deadband Variables
     byte output_deadband= 5;
 
    
  //Miscellaneous Variables
    int rot_offset=0;   //Offset from Vertical, System inherent error. Use if System always tends to run in one direction
    double angle;
  
  
//PID Setup
  //PID Variables 
    double rot_kp= 1.0;
    double rot_ki= 10.0;
    double rot_kd= 0.0;
    double y, Output, rot_Setpoint;
    double a = 0.70;
         
 //PID Controller initialisation
    PID rot_Controller(&angle,&Output,&rot_Setpoint,rot_kp,rot_ki,rot_kd,DIRECT);
  
 
void setup() {
  // PWM Frequenz fuer Timer 4 bleibt bei 490 Hz da 31kHz beim Getriebmotor zu Problem fhrt 
  int myEraser = 7;
  TCCR4B &= ~myEraser;  // Timerregister fuer timer 4 loeschen
  int myPrescaler = 3;  // Vorteiler bleibt auf Defaultwert 3 (=490Hz)       
  TCCR4B |= myPrescaler;  // Vorteiler laden
  // Ein / Ausgänge definieren
  pinMode(RPWM_Output, OUTPUT);
  pinMode(LPWM_Output, OUTPUT);
  analogWrite(LPWM_Output, 0);
  analogWrite(RPWM_Output, 0);  
  Serial.begin(115200); 
  pinMode(pin_photoint,INPUT);
 
//Initialize Encoder counts
  rotEnc.write(-4000); //set when pendulum is hanging straight down due to gravity
  
 //PID Settings
     rot_Controller.SetMode(AUTOMATIC);
     rot_Controller.SetOutputLimits(-100,100);  
     rot_Controller.SetSampleTime(4); //2
     rot_Controller.SetTunings(rot_kp,rot_ki,rot_kd);
 }

void loop() {
  angle = rotEnc.read();
  if (angle < limit_angle && angle > (limit_angle*-1)){    //perform as long as pendulum is within the angular limits  
                                 // Execute angular PID controller
                                 rot_Controller.Compute();
                                  y = a * y + (1.0 - a) * Output;
                                  Serial.println(y);
                                 // Serial.println(Output);        // bei Bedarf Ausgabe der Stellgroesse auf dem seriellen Plotter   
                                 if(Output > output_deadband){
                                     analogWrite(LPWM_Output, 0);
                                     analogWrite(RPWM_Output,  Output); 
                                 }     
                                 else if (Output < -output_deadband){
                                    analogWrite(RPWM_Output, 0);
                                    analogWrite(LPWM_Output,  abs(Output));
                                 }
                                 else {
                                    analogWrite(LPWM_Output, 0);
                                    analogWrite(RPWM_Output, 0);
                                    // Output=0;
                                }
          } 
     else {   //stop carrigage if pendulum has tipped out of the angular limits
                        analogWrite(LPWM_Output, 0);
                        analogWrite(RPWM_Output, 0);    
           };
 
 } //End Loop
 
